@@ -1,12 +1,14 @@
 # Copyright 2003-2008 by Leighton Pritchard.  All rights reserved.
 # Revisions copyright 2008-2017 by Peter Cock.
-# This code is part of the Biopython distribution and governed by its
-# license.  Please see the LICENSE file that should have been included
-# as part of this package.
 #
-# Contact:       Leighton Pritchard, Scottish Crop Research Institute,
+# This file is part of the Biopython distribution and governed by your
+# choice of the "Biopython License Agreement" or the "BSD 3-Clause License".
+# Please see the LICENSE file that should have been included as part of this
+# package.
+#
+# Contact:       Leighton Pritchard, The James Hutton Institute,
 #                Invergowrie, Dundee, Scotland, DD2 5DA, UK
-#                L.Pritchard@scri.ac.uk
+#                Leighton.Pritchard@hutton.ac.uk
 ################################################################################
 
 """AbstractDrawer module (considered to be a private module, the API may change!).
@@ -31,16 +33,13 @@ like SeqFeatures.
 """
 
 # ReportLab imports
-from __future__ import print_function
-
-from Bio._py3k import range
 
 from reportlab.lib import pagesizes
 from reportlab.lib import colors
 from reportlab.graphics.shapes import Polygon
 
 from math import pi, sin, cos
-
+from itertools import islice
 
 ################################################################################
 # METHODS
@@ -55,28 +54,29 @@ def page_sizes(size):
      - size - A string representing a standard page size, eg 'A4' or 'LETTER'
 
     """
-    sizes = {'A0': pagesizes.A0,    # ReportLab pagesizes, keyed by ISO string
-             'A1': pagesizes.A1,
-             'A2': pagesizes.A2,
-             'A3': pagesizes.A3,
-             'A4': pagesizes.A4,
-             'A5': pagesizes.A5,
-             'A6': pagesizes.A6,
-             'B0': pagesizes.B0,
-             'B1': pagesizes.B1,
-             'B2': pagesizes.B2,
-             'B3': pagesizes.B3,
-             'B4': pagesizes.B4,
-             'B5': pagesizes.B5,
-             'B6': pagesizes.B6,
-             'ELEVENSEVENTEEN': pagesizes.ELEVENSEVENTEEN,
-             'LEGAL': pagesizes.LEGAL,
-             'LETTER': pagesizes.LETTER
-             }
+    sizes = {  # ReportLab pagesizes, keyed by ISO string
+        "A0": pagesizes.A0,
+        "A1": pagesizes.A1,
+        "A2": pagesizes.A2,
+        "A3": pagesizes.A3,
+        "A4": pagesizes.A4,
+        "A5": pagesizes.A5,
+        "A6": pagesizes.A6,
+        "B0": pagesizes.B0,
+        "B1": pagesizes.B1,
+        "B2": pagesizes.B2,
+        "B3": pagesizes.B3,
+        "B4": pagesizes.B4,
+        "B5": pagesizes.B5,
+        "B6": pagesizes.B6,
+        "ELEVENSEVENTEEN": pagesizes.ELEVENSEVENTEEN,
+        "LEGAL": pagesizes.LEGAL,
+        "LETTER": pagesizes.LETTER,
+    }
     try:
         return sizes[size]
     except KeyError:
-        raise ValueError("%s not in list of page sizes" % size)
+        raise ValueError("%s not in list of page sizes" % size) from None
 
 
 def _stroke_and_fill_colors(color, border):
@@ -84,10 +84,11 @@ def _stroke_and_fill_colors(color, border):
     if not isinstance(color, colors.Color):
         raise ValueError("Invalid color %r" % color)
 
-    if color == colors.white and border is None:   # Force black border on
-        strokecolor = colors.black                 # white boxes with
-    elif border is None:                           # undefined border, else
-        strokecolor = color                        # use fill color
+    if color == colors.white and border is None:
+        # Force black border on white boxes with undefined border
+        strokecolor = colors.black
+    elif border is None:
+        strokecolor = color  # use fill color
     elif border:
         if not isinstance(border, colors.Color):
             raise ValueError("Invalid border color %r" % border)
@@ -99,9 +100,9 @@ def _stroke_and_fill_colors(color, border):
     return strokecolor, color
 
 
-def draw_box(point1, point2,
-             color=colors.lightgreen, border=None, colour=None,
-             **kwargs):
+def draw_box(
+    point1, point2, color=colors.lightgreen, border=None, colour=None, **kwargs
+):
     """Draw a box.
 
     Arguments:
@@ -125,15 +126,18 @@ def draw_box(point1, point2,
     strokecolor, color = _stroke_and_fill_colors(color, border)
 
     x1, y1, x2, y2 = min(x1, x2), min(y1, y2), max(x1, x2), max(y1, y2)
-    return Polygon([x1, y1, x2, y1, x2, y2, x1, y2],
-                   strokeColor=strokecolor,
-                   fillColor=color,
-                   strokewidth=0,
-                   **kwargs)
+    return Polygon(
+        [x1, y1, x2, y1, x2, y2, x1, y2],
+        strokeColor=strokecolor,
+        fillColor=color,
+        strokewidth=0,
+        **kwargs
+    )
 
 
-def draw_cut_corner_box(point1, point2, corner=0.5,
-                        color=colors.lightgreen, border=None, **kwargs):
+def draw_cut_corner_box(
+    point1, point2, corner=0.5, color=colors.lightgreen, border=None, **kwargs
+):
     """Draw a box with the corners cut off."""
     x1, y1 = point1
     x2, y2 = point2
@@ -150,24 +154,37 @@ def draw_cut_corner_box(point1, point2, corner=0.5,
     x_corner = min(boxheight * 0.5 * corner, boxwidth * 0.5)
     y_corner = min(boxheight * 0.5 * corner, boxheight * 0.5)
 
-    return Polygon([x1, y1 + y_corner,
-                    x1, y2 - y_corner,
-                    x1 + x_corner, y2,
-                    x2 - x_corner, y2,
-                    x2, y2 - y_corner,
-                    x2, y1 + y_corner,
-                    x2 - x_corner, y1,
-                    x1 + x_corner, y1],
-                   strokeColor=strokecolor,
-                   strokeWidth=1,
-                   strokeLineJoin=1,  # 1=round
-                   fillColor=color,
-                   **kwargs)
+    points = [
+        x1,
+        y1 + y_corner,
+        x1,
+        y2 - y_corner,
+        x1 + x_corner,
+        y2,
+        x2 - x_corner,
+        y2,
+        x2,
+        y2 - y_corner,
+        x2,
+        y1 + y_corner,
+        x2 - x_corner,
+        y1,
+        x1 + x_corner,
+        y1,
+    ]
+    return Polygon(
+        deduplicate(points),
+        strokeColor=strokecolor,
+        strokeWidth=1,
+        strokeLineJoin=1,  # 1=round
+        fillColor=color,
+        **kwargs
+    )
 
 
-def draw_polygon(list_of_points,
-                 color=colors.lightgreen, border=None, colour=None,
-                 **kwargs):
+def draw_polygon(
+    list_of_points, color=colors.lightgreen, border=None, colour=None, **kwargs
+):
     """Draw polygon.
 
     Arguments:
@@ -190,16 +207,26 @@ def draw_polygon(list_of_points,
         xy_list.append(x)
         xy_list.append(y)
 
-    return Polygon(xy_list,
-                   strokeColor=strokecolor,
-                   fillColor=color,
-                   strokewidth=0,
-                   **kwargs)
+    return Polygon(
+        deduplicate(xy_list),
+        strokeColor=strokecolor,
+        fillColor=color,
+        strokewidth=0,
+        **kwargs
+    )
 
 
-def draw_arrow(point1, point2, color=colors.lightgreen, border=None,
-               shaft_height_ratio=0.4, head_length_ratio=0.5, orientation='right',
-               colour=None, **kwargs):
+def draw_arrow(
+    point1,
+    point2,
+    color=colors.lightgreen,
+    border=None,
+    shaft_height_ratio=0.4,
+    head_length_ratio=0.5,
+    orientation="right",
+    colour=None,
+    **kwargs
+):
     """Draw an arrow.
 
     Returns a closed path object representing an arrow enclosed by the
@@ -228,13 +255,14 @@ def draw_arrow(point1, point2, color=colors.lightgreen, border=None,
     # using the same relative co-ordinates:
     xmin, ymin = min(x1, x2), min(y1, y2)
     xmax, ymax = max(x1, x2), max(y1, y2)
-    if orientation == 'right':
+    if orientation == "right":
         x1, x2, y1, y2 = xmin, xmax, ymin, ymax
-    elif orientation == 'left':
+    elif orientation == "left":
         x1, x2, y1, y2 = xmax, xmin, ymin, ymax
     else:
-        raise ValueError("Invalid orientation %s, should be 'left' or 'right'"
-                         % repr(orientation))
+        raise ValueError(
+            "Invalid orientation %s, should be 'left' or 'right'" % repr(orientation)
+        )
 
     # We define boxheight and boxwidth accordingly, and calculate the shaft
     # height from these.  We also ensure that the maximum head length is
@@ -250,20 +278,56 @@ def draw_arrow(point1, point2, color=colors.lightgreen, border=None,
     shaftbase = boxheight - shafttop
     headbase = boxwidth - headlength
     midheight = 0.5 * boxheight
-    return Polygon([x1, y1 + shafttop,
-                    x1 + headbase, y1 + shafttop,
-                    x1 + headbase, y2,
-                    x2, y1 + midheight,
-                    x1 + headbase, y1,
-                    x1 + headbase, y1 + shaftbase,
-                    x1, y1 + shaftbase],
-                   strokeColor=strokecolor,
-                   # strokeWidth=max(1, int(boxheight/40.)),
-                   strokeWidth=1,
-                   # default is mitre/miter which can stick out too much:
-                   strokeLineJoin=1,  # 1=round
-                   fillColor=color,
-                   **kwargs)
+
+    points = [
+        x1,
+        y1 + shafttop,
+        x1 + headbase,
+        y1 + shafttop,
+        x1 + headbase,
+        y2,
+        x2,
+        y1 + midheight,
+        x1 + headbase,
+        y1,
+        x1 + headbase,
+        y1 + shaftbase,
+        x1,
+        y1 + shaftbase,
+    ]
+
+    return Polygon(
+        deduplicate(points),
+        strokeColor=strokecolor,
+        # strokeWidth=max(1, int(boxheight/40.)),
+        strokeWidth=1,
+        # default is mitre/miter which can stick out too much:
+        strokeLineJoin=1,  # 1=round
+        fillColor=color,
+        **kwargs
+    )
+
+
+def deduplicate(points):
+    """Remove adjacent duplicate points.
+
+    This is important for use with the Polygon class since reportlab has a
+    bug with duplicate points.
+
+    Arguments:
+     - points - list of points [x1, y1, x2, y2,...]
+
+    Returns a list in the same format with consecutive duplicates removed
+    """
+    assert len(points) % 2 == 0
+    if len(points) < 2:
+        return points
+    newpoints = points[0:2]
+    for x, y in zip(islice(points, 2, None, 2), islice(points, 3, None, 2)):
+        if x != newpoints[-2] or y != newpoints[-1]:
+            newpoints.append(x)
+            newpoints.append(y)
+    return newpoints
 
 
 def angle2trig(theta):
@@ -278,7 +342,7 @@ def angle2trig(theta):
     """
     c = cos(theta * pi / 180)
     s = sin(theta * pi / 180)
-    return(c, s, -s, c)         # Vector for rotating point around an origin
+    return (c, s, -s, c)  # Vector for rotating point around an origin
 
 
 def intermediate_points(start, end, graph_data):
@@ -287,32 +351,34 @@ def intermediate_points(start, end, graph_data):
     Returns a list of (start, end, value) tuples describing the passed
     graph data as 'bins' between position midpoints.
     """
-    # print start, end, len(graph_data)
-    newdata = []    # data in form (X0, X1, val)
+    newdata = []  # data in form (X0, X1, val)
     # add first block
-    newdata.append((start,
-                    graph_data[0][0] + (graph_data[1][0] - graph_data[0][0]) / 2.,
-                    graph_data[0][1]))
+    newdata.append(
+        (
+            start,
+            graph_data[0][0] + (graph_data[1][0] - graph_data[0][0]) / 2.0,
+            graph_data[0][1],
+        )
+    )
     # add middle set
     for index in range(1, len(graph_data) - 1):
         lastxval, lastyval = graph_data[index - 1]
         xval, yval = graph_data[index]
         nextxval, nextyval = graph_data[index + 1]
-        newdata.append((lastxval + (xval - lastxval) / 2.,
-                        xval + (nextxval - xval) / 2., yval))
+        newdata.append(
+            (lastxval + (xval - lastxval) / 2.0, xval + (nextxval - xval) / 2.0, yval)
+        )
     # add last block
-    newdata.append((xval + (nextxval - xval) / 2.,
-                    end, graph_data[-1][1]))
-    # print newdata[-1]
-    # print newdata
+    newdata.append((xval + (nextxval - xval) / 2.0, end, graph_data[-1][1]))
     return newdata
 
+
 ################################################################################
 # CLASSES
 ################################################################################
 
 
-class AbstractDrawer(object):
+class AbstractDrawer:
     """Abstract Drawer.
 
     Attributes:
@@ -334,9 +400,22 @@ class AbstractDrawer(object):
 
     """
 
-    def __init__(self, parent, pagesize='A3', orientation='landscape',
-                 x=0.05, y=0.05, xl=None, xr=None, yt=None, yb=None,
-                 start=None, end=None, tracklines=0, cross_track_links=None):
+    def __init__(
+        self,
+        parent,
+        pagesize="A3",
+        orientation="landscape",
+        x=0.05,
+        y=0.05,
+        xl=None,
+        xr=None,
+        yt=None,
+        yb=None,
+        start=None,
+        end=None,
+        tracklines=0,
+        cross_track_links=None,
+    ):
         """Create the object.
 
         Arguments:
@@ -351,8 +430,6 @@ class AbstractDrawer(object):
            margins to the page
          - xl        Float (0->1) describing the relative size of the left X
            margin to the page (overrides x)
-         - xl        Float (0->1) describing the relative size of the left X
-           margin to the page (overrides x)
          - xr        Float (0->1) describing the relative size of the right X
            margin to the page (overrides x)
          - yt        Float (0->1) describing the relative size of the top Y
@@ -367,13 +444,13 @@ class AbstractDrawer(object):
            feature A, track B, feature B) to be linked.
 
         """
-        self._parent = parent   # The calling Diagram object
+        self._parent = parent  # The calling Diagram object
 
         # Perform 'administrative' tasks of setting up the page
-        self.set_page_size(pagesize, orientation)   # Set drawing size
-        self.set_margins(x, y, xl, xr, yt, yb)      # Set page margins
+        self.set_page_size(pagesize, orientation)  # Set drawing size
+        self.set_margins(x, y, xl, xr, yt, yb)  # Set page margins
         self.set_bounds(start, end)  # Set limits on what will be drawn
-        self.tracklines = tracklines    # Set flags
+        self.tracklines = tracklines  # Set flags
         if cross_track_links is None:
             cross_track_links = []
         else:
@@ -389,7 +466,7 @@ class AbstractDrawer(object):
          - orientation   String: 'landscape' or 'portrait'
 
         """
-        if isinstance(pagesize, str):     # A string, so translate
+        if isinstance(pagesize, str):  # A string, so translate
             pagesize = page_sizes(pagesize)
         elif isinstance(pagesize, tuple):  # A tuple, so don't translate
             pagesize = pagesize
@@ -398,9 +475,9 @@ class AbstractDrawer(object):
         shortside, longside = min(pagesize), max(pagesize)
 
         orientation = orientation.lower()
-        if orientation not in ('landscape', 'portrait'):
+        if orientation not in ("landscape", "portrait"):
             raise ValueError("Orientation %s not recognised" % orientation)
-        if orientation == 'landscape':
+        if orientation == "landscape":
             self.pagesize = (longside, shortside)
         else:
             self.pagesize = (shortside, longside)
@@ -428,10 +505,16 @@ class AbstractDrawer(object):
 
         # Set page limits, center and height/width
         self.x0, self.y0 = self.pagesize[0] * xmargin_l, self.pagesize[1] * ymargin_btm
-        self.xlim, self.ylim = self.pagesize[0] * (1 - xmargin_r), self.pagesize[1] * (1 - ymargin_top)
+        self.xlim, self.ylim = (
+            self.pagesize[0] * (1 - xmargin_r),
+            self.pagesize[1] * (1 - ymargin_top),
+        )
         self.pagewidth = self.xlim - self.x0
         self.pageheight = self.ylim - self.y0
-        self.xcenter, self.ycenter = self.x0 + self.pagewidth / 2., self.y0 + self.pageheight / 2.
+        self.xcenter, self.ycenter = (
+            self.x0 + self.pagewidth / 2.0,
+            self.y0 + self.pageheight / 2.0,
+        )
 
     def set_bounds(self, start, end):
         """Set start and end points for the drawing as a whole.
@@ -447,7 +530,7 @@ class AbstractDrawer(object):
             start, end = end, start
 
         if start is None or start < 0:  # Check validity of passed args and
-            start = 0   # default to 0
+            start = 0  # default to 0
         if end is None or end < 0:
             end = high + 1  # default to track range top limit
 
